//
// Article.C -- Manage newsgroup articles
//
// Copyright 2003 Michael Sweet
// Copyright 2002 Greg Ercolano
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public Licensse as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "Article.H"

// RETURN ASCII VERSION OF AN UNSIGNED LONG
static const char *ultoa(unsigned long num)
{
    static char s[80];
    sprintf(s, "%lu", num);
    return(s);
}

int Article::Load(const char *group, unsigned long num)
{
    number = num;

    // ZERO OUT FIELDS
    valid         = 0;		// assume invalid until successful
    filename[0]   = 0;
    messageid[0]  = 0;
    subject[0]    = 0;
    from[0]       = 0;
    date[0]       = 0;
    references[0] = 0;
    xref[0]       = 0;
    lines         = 0;

    if ( strlen(group) >= GROUP_MAX )
         { errmsg = "Group name too long"; return(-1); }

    if ( this->group != group )
        strcpy(this->group, group);

    snprintf(filename, sizeof(filename), "%s/%s/%lu", 
    	     (const char*)G_conf.SpoolDir(),
             (const char*)group, 
	     (unsigned long)number);

    // CONVERT NEWSGROUP NAMES TO FILE SYSTEM NAMES (rush.general -> rush/general)
    {
	char *ss;
	while ( ( ss = strchr(filename, '.') ) != NULL )
	    { *ss = '/'; }
    }

    FILE *fp = fopen(filename, "r");
    if ( fp == NULL )
    {
        errmsg = "article ";
	errmsg.append(ultoa(number));
	errmsg.append(" no longer exists: '");
	errmsg.append(filename);
	errmsg.append("': ");
	errmsg.append(strerror(errno));
	return(-1);
    }

    // LOAD KEY/VALUE PAIRS
    char s[LINE_LEN];
    while ( fgets(s, sizeof(s)-1, fp) != NULL )
    {
	// END OF HEADER?
	if ( s[0] == '\n' ) { break; }

	char *key = s, *val = 0;
	for ( char *ss = s; *ss; ss++ )
	{
	    if ( *ss == '\n' )
		{ *ss = 0; break; }
	    else if ( *ss == ' ' && val == 0 )
		{ *ss = 0; val = ++ss; }
	}

	// fprintf(stderr, "KEY='%s' VAL='%s'\n", key, val);
	if ( val )
	{
	    // TRUNCATE IF TOO LONG
	    if ( strlen(val) >= FIELD_MAX )
	        { val[FIELD_MAX-1] = 0; }

	    // HEADER NAMES ARE CASE INSENSITIVE: INTERNET DRAFT (Son of RFC1036)
	    if ( strcasecmp(key, "Subject:" ) == 0 ) 
	        { strcpy(subject, val); }
	    else if ( strcasecmp(key, "From:" ) == 0 )
	        { strcpy(from, val); }
	    else if ( strcasecmp(key, "Date:" ) == 0 )
	        { strcpy(date, val); }
	    else if ( strcasecmp(key, "Xref:" ) == 0 )
	        { strcpy(xref, val); }
	    else if ( strcasecmp(key, "Message-ID:") == 0 ) 
	        { strcpy(messageid, val); }
	    else if ( strcasecmp(key, "References:") == 0 ) 
	        { strcpy(references, val); }
	    else if ( strcasecmp(key, "Lines:") == 0 ) 
	        { lines = atoi(val); }
	}
    }
    fclose(fp);

    if ( messageid == "" ) { errmsg = "No 'Message-ID' field"; return(-1); }
    if ( from == ""      ) { errmsg = "No 'From' field";       return(-1); }

    valid = 1;
    return(0);
}

int Article::Load(unsigned long num)
{
    if ( group[0] == 0 )
	{ errmsg = "No group selected"; return(-1); }

    return(Load(group, num));
}

// SEND ARTICLE TO REMOTE VIA FD
//    head: 1=send header
//    body: 1=send body
//    If both head and body are 1, separator (blank line)
//    is also sent.
//
int Article::SendArticle(int fd, int head, int body)
{
    FILE *fp = fopen(filename, "r");
    if ( fp == NULL )
    {
        errmsg = "article ";
	errmsg.append(ultoa(number));
	errmsg.append(" no longer exists: ");
	errmsg.append(strerror(errno));
	return(-1);
    }

    char s[LINE_LEN];
    enum Mode { MODE_HEAD, MODE_SEP, MODE_BODY };
    Mode mode = MODE_HEAD;

    while ( fgets(s, LINE_LEN-2, fp) )
    {
        if ( mode == MODE_SEP )
	{
	    // MOVED OFF SEPARATOR INTO BODY
	    mode = MODE_BODY;
	}
	else if ( s[0] == '\n' && mode == MODE_HEAD )
	{
	    // END OF HEADER, AND NOT SENDING BODY? DONE
	    mode = MODE_SEP;			// end of header
	    if ( body == 0 ) { break; }		// not sending body? done
	}

	// LINE TOO LONG? -- TRUNCATE
	s[LINE_LEN-4] = '\n';
	s[LINE_LEN-3] = 0;

        char *ss = strchr(s, '\n');
	if ( ss )
	{
	    // TERMINATE WITH CRLF
	    *ss = '\r'; ss++;
	    *ss = '\n'; ss++;
	    *ss = 0;
	}

	if ( ( mode == MODE_HEAD && head ) ||
	     ( mode == MODE_BODY && body ) ||
	     ( mode == MODE_SEP && head && body ) )
	{
	    write(fd, s, strlen(s));
	    G_conf.LogMessage(L_DEBUG, "SEND: %s", s);
	}
    }
    fclose(fp);
    return(0);
}

int Article::SendHead(int fd)
{
    return(SendArticle(fd, 1, 0));
}

int Article::SendBody(int fd)
{
    return(SendArticle(fd, 0, 1));
}

void Article::Overview(string &reply, const char *overview[] )
{
    reply = ultoa(Number());
    for ( int r=0; overview[r]; r++ )
    {
	// HEADER NAMES ARE CASE INSENSITIVE: INTERNET DRAFT (Son of RFC1036)
	     if (!strcasecmp(overview[r], "Subject:"   ))
	{
	    reply += "\t";
	    reply += subject;
	}
	else if (!strcasecmp(overview[r], "From:"      ))
	{
	    reply += "\t";
	    reply += from;
	}
	else if (!strcasecmp(overview[r], "Date:"      ))
	{
	    reply += "\t";
	    reply += date;
	}
	else if (!strcasecmp(overview[r], "Message-ID:"))
	{
	    reply += "\t";
	    reply += messageid;
	}
	else if (!strcasecmp(overview[r], "References:"))
	{
	    reply += "\t";
	    reply += references;
	}
	else if (!strcasecmp(overview[r], "Lines:"     ))
	{
	    reply += "\t"; 
	    if ( lines > 0 )
		{ reply += ultoa(lines); } 
	}
	else if (!strcasecmp(overview[r], "Bytes:" ))
	{
	    // WE DONT KEEP TRACK OF BYTES -- LEAVE FIELD EMPTY
	    reply += "\t";
	}
	else if (!strcasecmp(overview[r], "Xref:full"))
	{
	    reply += "\t"; 
	    if ( xref[0] != 0 )
		{ reply += "Xref: "; reply += xref; }
	}
    }
    reply += '\0';
}
